import { declare } from '@babel/helper-plugin-utils';
import syntaxDynamicImport from '@babel/plugin-syntax-dynamic-import';
import syntaxImportMeta from '@babel/plugin-syntax-import-meta';
import transformClassProperties from '@babel/plugin-proposal-class-properties';
import transformJsonStrings from '@babel/plugin-proposal-json-strings';
import env from '@babel/preset-env';
import react from '@babel/preset-react';
import typescript from '@babel/preset-typescript';
import pluginFilterImports from 'babel-plugin-filter-imports';
import pluginTransformImports from 'babel-plugin-transform-imports';
import pluginStyledComponents from 'babel-plugin-styled-components';
import pluginDevExpression from 'babel-plugin-dev-expression';
import pluginTransformRemoveConsole from 'babel-plugin-transform-remove-console';
import pluginTransformRuntime from '@babel/plugin-transform-runtime';

import { TransformOptions } from '@babel/core';

export default declare(
  (
    api,
    {
      node,
      cherryPick = true,
      debug,
      ssr,
      useBuiltIns = 'usage',
      minifyStyledComponents,
      modules = false,
      excludeImports,
      removeConsoleLogs = false
    }
  ): TransformOptions => {
    api.assertVersion(7);
    const babelEnv = api.env();

    // TODO: type this guy
    const envOpts: any = {
      modules: babelEnv === 'test' ? 'cjs' : modules,
      useBuiltIns,
      targets: {
        browsers: [
          'last 7 Chrome version',
          'last 7 Firefox version',
          'last 2 Edge version',
          'last 1 Safari version',
          'last 1 Android version',
          'last 1 ChromeAndroid version',
          'last 1 FirefoxAndroid version',
          'last 1 iOS version',
          'last 2 Samsung version',
          // hope we get to delete the following line one day!
          'IE >= 11'
        ]
      }
    };

    if (node) {
      envOpts.targets.node = node;
      // If the node option is set then we are not interested in browser polyfills.
      delete envOpts.targets.browsers;
    }

    return {
      plugins: [
        // https://babeljs.io/docs/en/babel-plugin-transform-remove-console
        removeConsoleLogs && [
          pluginTransformRemoveConsole,
          {
            exclude: ['error', 'warn']
          }
        ],
        // https://babeljs.io/docs/en/babel-plugin-transform-runtime
        // we only want to include polyfills when we are not running jest
        babelEnv !== 'test' && [
          pluginTransformRuntime,
          {
            corejs: 2,
            helpers: true,
            regenerator: true,
            useESModules: envOpts.modules === false
          }
        ],
        // https://github.com/4Catalyzer/babel-plugin-dev-expression
        pluginDevExpression,
        syntaxDynamicImport,
        syntaxImportMeta,
        [transformClassProperties, { loose: false }],
        transformJsonStrings,
        cherryPick && babelEnv !== 'test'
          ? [
              /*
            babel-plugin-lodash
            https://github.com/lodash/babel-plugin-lodash#babel-plugin-lodash

            A simple transform to cherry-pick Lodash/Recompose modules
            so you don’t have to.

            Cherry picking ensures that the whole module is not
            imported resulting smalling bundles in your app.

            https://github.com/acdlite/recompose#optimizing-bundle-size
            https://www.blazemeter.com/blog/the-correct-way-to-import-lodash-libraries-a-benchmark

            Transforms

            import _ from 'lodash' import { add } from 'lodash/fp'

            const addOne = add(1) _.map([1, 2, 3], addOne)

            roughly to

            import _add from 'lodash/fp/add' import _map from
            'lodash/map'

            const addOne = _add(1) _map([1, 2, 3], addOne) */

              'lodash',
              {
                id: ['lodash', 'recompose']
              }
            ]
          : null,
        cherryPick && babelEnv !== 'test'
          ? [
              /*
            babel-plugin-transform-imports

            Transforms member style imports nto default style import.

            This cherry picking ensures that the whole module is not
            imported - resulting smalling bundles in your app.

            https://github.com/lodash/babel-plugin-lodash#babel-plugin-lodash

            https://github.com/acdlite/recompose#optimizing-bundle-size

            https://www.blazemeter.com/blog/the-correct-way-to-import-lodash-libraries-a-benchmark

            Transforms member style imports:

            import { Row, Grid as MyGrid } from 'react-bootstrap';
            import { merge } from 'lodash';
            ...into default style imports:

            import Row from 'react-bootstrap/lib/Row';
            import MyGrid from 'react-bootstrap/lib/Grid';
            import merge from 'lodash/merge'; */

              pluginTransformImports,
              {
                'redux-form': {
                  transform: `redux-form/${
                    babelEnv === 'test' ? 'lib' : 'es'
                  }/\${member}`,
                  preventFullImport: true,
                  skipDefaultConversion: false
                },
                '@origin-digital/recompose': {
                  transform: `@origin-digital/recompose/dist/es/\${member}`,
                  preventFullImport: true,
                  skipDefaultConversion: true
                },
                '@origin-digital/validation': {
                  transform: `@origin-digital/validation/dist/es/\${member}`,
                  preventFullImport: false,
                  skipDefaultConversion: true
                },
                '@origin-digital/normalize': {
                  transform: `@origin-digital/normalize/dist/es/\${member}`,
                  preventFullImport: false,
                  skipDefaultConversion: true
                },
                '@origin-digital/style-guide-legacy': {
                  transform: `@origin-digital/style-guide-legacy/dist/es/\${member}`,
                  preventFullImport: false,
                  skipDefaultConversion: false
                },
                '@origin-digital/utils': {
                  transform: `@origin-digital/utils/dist/es/\${member}`,
                  preventFullImport: true,
                  skipDefaultConversion: true
                },
                'react-virtualized': {
                  transform: `react-virtualized/dist/${
                    babelEnv === 'test' ? 'commonjs' : 'es'
                  }/\${member}`,
                  preventFullImport: true,
                  skipDefaultConversion: false
                }
              }
            ]
          : null,
        excludeImports
          ? [
              pluginFilterImports,
              {
                imports: excludeImports
              }
            ]
          : null,
        [
          /*
          babel-plugin-styled-components
          https://github.com/styled-components/babel-plugin-styled-components#babel-plugin-styled-components

          This plugin is a highly recommended supplement to the base
          styled-components library, offering some useful features:

          consistently hashed component classNames between
          environments (a must for server-side rendering)

          automatic library usage tweaks that allow for some bytes to be dropped
          from your bundle

          better debugging through automatic annotation of your styled components based on their context
          in the file system, etc.

          various types of minification for styles and the tagged template
          literals styled-components uses
          */
          pluginStyledComponents,
          {
            debug,
            uglifyPure: true,
            minify: minifyStyledComponents,
            ssr
          }
        ]
      ].filter(Boolean),
      presets: [[env, envOpts], react, typescript]
    };
  }
);
